@@ -1,12 +1,13 @@
 from dataclasses import dataclass, field
 from enum import Enum
 from typing import Any, Union, List
-from xmlrpc.client import DateTime
+from datetime import datetime
 
 from .super_dtos import PagenatedResponse
 from .transcription_dtos import GetTranscription
 from .region_dtos import GetRegionType
 from .line_dtos import GetLineType
+from .utils import iso_string_date_to_datetime
 
 
 class ReadDirection(str, Enum):
@@ -49,8 +50,8 @@ class GetDocument:
     read_direction: ReadDirection
     line_offset: LineOffset
     parts_count: int
-    created_at: DateTime
-    updated_at: DateTime
+    created_at: datetime
+    updated_at: datetime
     transcriptions: List[GetTranscription] = field(default_factory=list)
     valid_block_types: List[GetRegionType] = field(default_factory=list)
     valid_line_types: List[GetLineType] = field(default_factory=list)
@@ -79,8 +80,8 @@ class GetDocument:
         self.read_direction = ReadDirection(read_direction)
         self.line_offset = LineOffset(line_offset)
         self.parts_count = parts_count
-        self.created_at = DateTime(created_at)
-        self.updated_at = DateTime(updated_at)
+        self.created_at = iso_string_date_to_datetime(created_at)
+        self.updated_at = iso_string_date_to_datetime(updated_at)
         self.transcriptions = [GetTranscription(**x) for x in transcriptions]
         self.valid_block_types = [GetRegionType(**x) for x in valid_block_types]
         self.valid_line_types = [GetLineType(**x) for x in valid_line_types]
